@@ -26,6 +26,7 @@
 # Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2020 Ram Rachum <ram@rachum.com>
 # Copyright (c) 2020 Anthony Sottile <asottile@umich.edu>
+# Copyright (c) 2021 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
 # Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
@@ -34,9 +35,10 @@
 """Checks for various exception related errors."""
 import builtins
 import inspect
-import typing
+from typing import Any, List, Optional
 
 import astroid
+from astroid import nodes, objects
 
 from pylint import checkers, interfaces
 from pylint.checkers import utils
@@ -57,7 +59,7 @@ def _annotated_unpack_infer(stmt, context=None):
     Returns an iterator which yields tuples in the format
     ('original node', 'inferred node').
     """
-    if isinstance(stmt, (astroid.List, astroid.Tuple)):
+    if isinstance(stmt, (nodes.List, nodes.Tuple)):
         for elt in stmt.elts:
             inferred = utils.safe_infer(elt)
             if inferred and inferred is not astroid.Uninferable:
@@ -69,10 +71,10 @@ def _annotated_unpack_infer(stmt, context=None):
         yield stmt, inferred
 
 
-def _is_raising(body: typing.List) -> bool:
+def _is_raising(body: List) -> bool:
     """Return true if the given statement node raise an exception"""
     for node in body:
-        if isinstance(node, astroid.Raise):
+        if isinstance(node, nodes.Raise):
             return True
     return False
 
@@ -200,26 +202,26 @@ class BaseVisitor:
         else:
             self.visit_default(node)
 
-    def visit_default(self, node):  # pylint: disable=unused-argument
+    def visit_default(self, _: nodes.NodeNG) -> None:
         """Default implementation for all the nodes."""
 
 
 class ExceptionRaiseRefVisitor(BaseVisitor):
     """Visit references (anything that is not an AST leaf)."""
 
-    def visit_name(self, name):
-        if name.name == "NotImplemented":
+    def visit_name(self, node: nodes.Name) -> None:
+        if node.name == "NotImplemented":
             self._checker.add_message("notimplemented-raised", node=self._node)
 
-    def visit_call(self, call):
-        if isinstance(call.func, astroid.Name):
-            self.visit_name(call.func)
+    def visit_call(self, node: nodes.Call) -> None:
+        if isinstance(node.func, nodes.Name):
+            self.visit_name(node.func)
         if (
-            len(call.args) > 1
-            and isinstance(call.args[0], astroid.Const)
-            and isinstance(call.args[0].value, str)
+            len(node.args) > 1
+            and isinstance(node.args[0], nodes.Const)
+            and isinstance(node.args[0].value, str)
         ):
-            msg = call.args[0].value
+            msg = node.args[0].value
             if "%" in msg or ("{" in msg and "}" in msg):
                 self._checker.add_message("raising-format-tuple", node=self._node)
 
@@ -227,14 +229,12 @@ class ExceptionRaiseRefVisitor(BaseVisitor):
 class ExceptionRaiseLeafVisitor(BaseVisitor):
     """Visitor for handling leaf kinds of a raise value."""
 
-    def visit_const(self, const):
-        if not isinstance(const.value, str):
-            # raising-string will be emitted from python3 porting checker.
-            self._checker.add_message(
-                "raising-bad-type", node=self._node, args=const.value.__class__.__name__
-            )
+    def visit_const(self, node: nodes.Const) -> None:
+        self._checker.add_message(
+            "raising-bad-type", node=self._node, args=node.value.__class__.__name__
+        )
 
-    def visit_instance(self, instance):
+    def visit_instance(self, instance: objects.ExceptionInstance) -> None:
         # pylint: disable=protected-access
         cls = instance._proxied
         self.visit_classdef(cls)
@@ -242,15 +242,15 @@ class ExceptionRaiseLeafVisitor(BaseVisitor):
     # Exception instances have a particular class type
     visit_exceptioninstance = visit_instance
 
-    def visit_classdef(self, cls):
-        if not utils.inherit_from_std_ex(cls) and utils.has_known_bases(cls):
-            if cls.newstyle:
+    def visit_classdef(self, node: nodes.ClassDef) -> None:
+        if not utils.inherit_from_std_ex(node) and utils.has_known_bases(node):
+            if node.newstyle:
                 self._checker.add_message("raising-non-exception", node=self._node)
 
-    def visit_tuple(self, _):
+    def visit_tuple(self, _: nodes.Tuple) -> None:
         self._checker.add_message("raising-bad-type", node=self._node, args="tuple")
 
-    def visit_default(self, node):
+    def visit_default(self, node: nodes.NodeNG) -> None:
         name = getattr(node, "name", node.__class__.__name__)
         self._checker.add_message("raising-bad-type", node=self._node, args=name)
 
@@ -270,7 +270,7 @@ class ExceptionsChecker(checkers.BaseChecker):
                 "default": OVERGENERAL_EXCEPTIONS,
                 "type": "csv",
                 "metavar": "<comma-separated class names>",
-                "help": "Exceptions that will emit a warning "
+                "help": "Exceptions that will emit a warning "  # pylint: disable=consider-using-f-string
                 'when being caught. Defaults to "%s".'
                 % (", ".join(OVERGENERAL_EXCEPTIONS),),
             },
@@ -290,7 +290,7 @@ class ExceptionsChecker(checkers.BaseChecker):
         "raising-format-tuple",
         "raise-missing-from",
     )
-    def visit_raise(self, node):
+    def visit_raise(self, node: nodes.Raise) -> None:
         if node.exc is None:
             self._check_misplaced_bare_raise(node)
             return
@@ -315,7 +315,7 @@ class ExceptionsChecker(checkers.BaseChecker):
         # Filter out if it's present in __exit__.
         scope = node.scope()
         if (
-            isinstance(scope, astroid.FunctionDef)
+            isinstance(scope, nodes.FunctionDef)
             and scope.is_method()
             and scope.name == "__exit__"
         ):
@@ -324,15 +324,15 @@ class ExceptionsChecker(checkers.BaseChecker):
         current = node
         # Stop when a new scope is generated or when the raise
         # statement is found inside a TryFinally.
-        ignores = (astroid.ExceptHandler, astroid.FunctionDef)
+        ignores = (nodes.ExceptHandler, nodes.FunctionDef)
         while current and not isinstance(current.parent, ignores):
             current = current.parent
 
-        expected = (astroid.ExceptHandler,)
+        expected = (nodes.ExceptHandler,)
         if not current or not isinstance(current.parent, expected):
             self.add_message("misplaced-bare-raise", node=node)
 
-    def _check_bad_exception_context(self, node: astroid.Raise) -> None:
+    def _check_bad_exception_context(self, node: nodes.Raise) -> None:
         """Verify that the exception context is properly set.
 
         An exception context can be only `None` or an exception.
@@ -341,15 +341,15 @@ class ExceptionsChecker(checkers.BaseChecker):
         if cause in (astroid.Uninferable, None):
             return
 
-        if isinstance(cause, astroid.Const):
+        if isinstance(cause, nodes.Const):
             if cause.value is not None:
                 self.add_message("bad-exception-context", node=node)
-        elif not isinstance(cause, astroid.ClassDef) and not utils.inherit_from_std_ex(
+        elif not isinstance(cause, nodes.ClassDef) and not utils.inherit_from_std_ex(
             cause
         ):
             self.add_message("bad-exception-context", node=node)
 
-    def _check_raise_missing_from(self, node: astroid.Raise) -> None:
+    def _check_raise_missing_from(self, node: nodes.Raise) -> None:
         if node.exc is None:
             # This is a plain `raise`, raising the previously-caught exception. No need for a
             # cause.
@@ -368,20 +368,18 @@ class ExceptionsChecker(checkers.BaseChecker):
             # The `except` doesn't have an `as exception:` part, meaning there's no way that
             # the `raise` is raising the same exception.
             self.add_message("raise-missing-from", node=node)
-        elif isinstance(node.exc, astroid.Call) and isinstance(
-            node.exc.func, astroid.Name
-        ):
+        elif isinstance(node.exc, nodes.Call) and isinstance(node.exc.func, nodes.Name):
             # We have a `raise SomeException(whatever)`.
             self.add_message("raise-missing-from", node=node)
         elif (
-            isinstance(node.exc, astroid.Name)
+            isinstance(node.exc, nodes.Name)
             and node.exc.name != containing_except_node.name.name
         ):
             # We have a `raise SomeException`.
             self.add_message("raise-missing-from", node=node)
 
     def _check_catching_non_exception(self, handler, exc, part):
-        if isinstance(exc, astroid.Tuple):
+        if isinstance(exc, nodes.Tuple):
             # Check if it is a tuple of exceptions.
             inferred = [utils.safe_infer(elt) for elt in exc.elts]
             if any(node is astroid.Uninferable for node in inferred):
@@ -394,14 +392,13 @@ class ExceptionsChecker(checkers.BaseChecker):
             ):
                 return
 
-        if not isinstance(exc, astroid.ClassDef):
+        if not isinstance(exc, nodes.ClassDef):
             # Don't emit the warning if the inferred stmt
             # is None, but the exception handler is something else,
             # maybe it was redefined.
-            if isinstance(exc, astroid.Const) and exc.value is None:
+            if isinstance(exc, nodes.Const) and exc.value is None:
                 if (
-                    isinstance(handler.type, astroid.Const)
-                    and handler.type.value is None
+                    isinstance(handler.type, nodes.Const) and handler.type.value is None
                 ) or handler.type.parent_of(exc):
                     # If the exception handler catches None or
                     # the exception component, which is None, is
@@ -432,16 +429,16 @@ class ExceptionsChecker(checkers.BaseChecker):
     def _check_try_except_raise(self, node):
         def gather_exceptions_from_handler(
             handler,
-        ) -> typing.Optional[typing.List[astroid.node_classes.NodeNG]]:
-            exceptions: typing.List[astroid.node_classes.NodeNG] = []
+        ) -> Optional[List[nodes.NodeNG]]:
+            exceptions: List[nodes.NodeNG] = []
             if handler.type:
                 exceptions_in_handler = utils.safe_infer(handler.type)
-                if isinstance(exceptions_in_handler, astroid.Tuple):
+                if isinstance(exceptions_in_handler, nodes.Tuple):
                     exceptions = list(
                         {
                             exception
                             for exception in exceptions_in_handler.elts
-                            if isinstance(exception, astroid.Name)
+                            if isinstance(exception, nodes.Name)
                         }
                     )
                 elif exceptions_in_handler:
@@ -487,23 +484,17 @@ class ExceptionsChecker(checkers.BaseChecker):
                 self.add_message("try-except-raise", node=handler_having_bare_raise)
 
     @utils.check_messages("wrong-exception-operation")
-    def visit_binop(self, node):
-        if isinstance(node.parent, astroid.ExceptHandler):
+    def visit_binop(self, node: nodes.BinOp) -> None:
+        if isinstance(node.parent, nodes.ExceptHandler):
             # except (V | A)
-            suggestion = "Did you mean '({}, {})' instead?".format(
-                node.left.as_string(),
-                node.right.as_string(),
-            )
+            suggestion = f"Did you mean '({node.left.as_string()}, {node.right.as_string()})' instead?"
             self.add_message("wrong-exception-operation", node=node, args=(suggestion,))
 
     @utils.check_messages("wrong-exception-operation")
-    def visit_compare(self, node):
-        if isinstance(node.parent, astroid.ExceptHandler):
+    def visit_compare(self, node: nodes.Compare) -> None:
+        if isinstance(node.parent, nodes.ExceptHandler):
             # except (V < A)
-            suggestion = "Did you mean '({}, {})' instead?".format(
-                node.left.as_string(),
-                ", ".join(operand.as_string() for _, operand in node.ops),
-            )
+            suggestion = f"Did you mean '({node.left.as_string()}, {', '.join(operand.as_string() for _, operand in node.ops)})' instead?"
             self.add_message("wrong-exception-operation", node=node, args=(suggestion,))
 
     @utils.check_messages(
@@ -515,10 +506,10 @@ class ExceptionsChecker(checkers.BaseChecker):
         "catching-non-exception",
         "duplicate-except",
     )
-    def visit_tryexcept(self, node):
+    def visit_tryexcept(self, node: nodes.TryExcept) -> None:
         """check for empty except"""
         self._check_try_except_raise(node)
-        exceptions_classes = []
+        exceptions_classes: List[Any] = []
         nb_handlers = len(node.handlers)
         for index, handler in enumerate(node.handlers):
             if handler.type is None:
@@ -531,7 +522,7 @@ class ExceptionsChecker(checkers.BaseChecker):
                     msg = "empty except clause should always appear last"
                     self.add_message("bad-except-order", node=node, args=msg)
 
-            elif isinstance(handler.type, astroid.BoolOp):
+            elif isinstance(handler.type, nodes.BoolOp):
                 self.add_message(
                     "binary-op-exception", node=handler, args=handler.type.op
                 )
@@ -552,21 +543,18 @@ class ExceptionsChecker(checkers.BaseChecker):
 
                     self._check_catching_non_exception(handler, exc, part)
 
-                    if not isinstance(exc, astroid.ClassDef):
+                    if not isinstance(exc, nodes.ClassDef):
                         continue
 
                     exc_ancestors = [
                         anc
                         for anc in exc.ancestors()
-                        if isinstance(anc, astroid.ClassDef)
+                        if isinstance(anc, nodes.ClassDef)
                     ]
 
                     for previous_exc in exceptions_classes:
                         if previous_exc in exc_ancestors:
-                            msg = "{} is an ancestor class of {}".format(
-                                previous_exc.name,
-                                exc.name,
-                            )
+                            msg = f"{previous_exc.name} is an ancestor class of {exc.name}"
                             self.add_message(
                                 "bad-except-order", node=handler.type, args=msg
                             )
