@@ -1,21 +1,25 @@
-# -*- coding: utf-8 -*-
 # Copyright (c) 2015 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
-# Copyright (c) 2016-2017 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2016-2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2016 Glenn Matthews <glmatthe@cisco.com>
-# Copyright (c) 2018 Ville Skyttä <ville.skytta@upcloud.com>
+# Copyright (c) 2018 Ville Skyttä <ville.skytta@iki.fi>
+# Copyright (c) 2019-2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2020 Anthony Sottile <asottile@umich.edu>
+# Copyright (c) 2021 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+
+from astroid import nodes
 
-import astroid
 from pylint.checkers import BaseTokenChecker
 from pylint.checkers.utils import check_messages
-from pylint.interfaces import ITokenChecker, IAstroidChecker
+from pylint.interfaces import IAstroidChecker, ITokenChecker
 
 
 class ElseifUsedChecker(BaseTokenChecker):
-    """Checks for use of "else if" when an "elif" could be used
-    """
+    """Checks for use of "else if" when an "elif" could be used"""
 
     __implements__ = (ITokenChecker, IAstroidChecker)
     name = "else_if_used"
@@ -45,18 +49,20 @@ class ElseifUsedChecker(BaseTokenChecker):
             elif token == "if":
                 self._elifs.append(False)
 
-    def leave_module(self, _):
+    def leave_module(self, _: nodes.Module) -> None:
         self._init()
 
-    def visit_ifexp(self, _):
+    def visit_ifexp(self, node: nodes.IfExp) -> None:
+        if isinstance(node.parent, nodes.FormattedValue):
+            return
         self._if_counter += 1
 
-    def visit_comprehension(self, node):
+    def visit_comprehension(self, node: nodes.Comprehension) -> None:
         self._if_counter += len(node.ifs)
 
     @check_messages("else-if-used")
-    def visit_if(self, node):
-        if isinstance(node.parent, astroid.If):
+    def visit_if(self, node: nodes.If) -> None:
+        if isinstance(node.parent, nodes.If):
             orelse = node.parent.orelse
             # current if node must directly follow an "else"
             if orelse and orelse == [node]:
